/*
    Bantumi
    Copyright 2005, 2006 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsj
    martin@martin.st
*/

#include "btconn.h"
#import "BluetoothController.h"


BTConnection::BTConnection(BluetoothController *c) {
	controller = c;
	deleted = false;
	bufferPos = 0;
}

BTConnection::~BTConnection() {

	deleted = true;
	if (!controller->released) {
		[controller shutdown];
		[controller release];
	}

}

int BTConnection::ready(const char **errString) {
	if ([controller->channel isOpen] && !controller->closed)
		return 1;
	*errString = "Connection closed";
	return -1;
}

bool BTConnection::read(const char **errString) {
	if (cb && bufferPos > 0) {
		cb->received(initialBuffer, bufferPos);
		bufferPos = 0;
	}
	NSEvent *event;
	while ((event = [NSApp nextEventMatchingMask: NSAnyEventMask untilDate: [NSDate distantPast] inMode: NSDefaultRunLoopMode dequeue: YES]) != nil) {
		[NSApp sendEvent: event];
	}
	return true;
}

bool BTConnection::write(const unsigned char *ptr, int n, const char **errString) {
	int mtu = [controller->channel getMTU];
	while (n > 0) {
		int min = n;
		if (mtu < min)
			min = mtu;
		IOReturn ret = [controller->channel writeSync: (void*) ptr length: min];
		if (ret != kIOReturnSuccess) {
			*errString = "Send error";
			return false;
		}
		ptr += min;
		n -= min;
	}
	return true;
}

void BTConnection::received(const unsigned char *ptr, int n) {
	if (cb) {
		if (bufferPos > 0) {
			cb->received(initialBuffer, bufferPos);
			bufferPos = 0;
		}
		cb->received(ptr, n);
	} else {
		int bufferSize = sizeof(initialBuffer)/sizeof(initialBuffer[0]);
		if (n + bufferPos > bufferSize)
			n = bufferSize - bufferPos;
		memcpy(initialBuffer + bufferPos, ptr, n);
	}
}



