/*
    Bantumi
    Copyright 2005, 2006 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsj
    martin@martin.st
*/

#include "connectwaitcontainer.h"
#include "connection.h"
#include "gui.h"
#include "symbianbt.h"
#include <bantumigl.rsg>
#include <aknnotewrappers.h>
#include <flogger.h>
#include "bantumiappview.h"
#include "bantumiappui.h"
#include <textresolver.h>
#include <charconv.h>
#include <string.h>

// don't call this until we're actually returning to the main game loop
// otherwise the main callback can be called from these event loops
void restartTimer() {
	CBantumiAppUi *appui = static_cast<CBantumiAppUi*>(CEikonEnv::Static()->AppUi());
	CBantumiAppView *appview = appui->iAppView;
	appview->SetForeground(ETrue);
}

void showWarningL(const char *str) {
	CCnvCharacterSetConverter *conv = CCnvCharacterSetConverter::NewLC();
	conv->PrepareToConvertToOrFromL(KCharacterSetIdentifierUtf8, CEikonEnv::Static()->FsSession());
	TInt state = CCnvCharacterSetConverter::KStateDefault;

	int len = strlen(str);
	HBufC8 *buf8 = HBufC8::NewLC(len+1);
	TPtr8 ptr8 = buf8->Des();
	while (*str)
		ptr8.Append(*str++);
	HBufC *buf = HBufC::NewLC(len);
	TPtr ptr = buf->Des();
	conv->ConvertToUnicode(ptr, ptr8, state);

	CAknWarningNote* note = new(ELeave) CAknWarningNote(ETrue);
	note->ExecuteLD(ptr);
	CleanupStack::PopAndDestroy(buf);
	CleanupStack::PopAndDestroy(buf8);
	CleanupStack::PopAndDestroy(conv);
}

void internalShowWarning(const char *str) {
	TRAPD(err, showWarningL(str));
}

void showWarning(const char *str) {
	internalShowWarning(str);
	restartTimer();
}

void showWarningL(TInt aError) {
	CTextResolver* resolver = CTextResolver::NewLC();
#ifdef EKA2
	TPtrC err = resolver->ResolveErrorString(aError);
#else
	TPtrC err = resolver->ResolveError(aError);
#endif
	CAknWarningNote* note = new(ELeave) CAknWarningNote(ETrue);
	note->ExecuteLD(err);
	CleanupStack::PopAndDestroy(resolver);
}

void internalShowWarning(TInt aError) {
	TRAPD(err, showWarningL(aError));
}



CConnectWaitContainer* CConnectWaitContainer::NewL(CBTConnection *aConn) {
	CConnectWaitContainer* self = new(ELeave) CConnectWaitContainer(aConn);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

CConnectWaitContainer::CConnectWaitContainer(CBTConnection *aConn) {
	iConn = aConn;
}

CConnectWaitContainer::~CConnectWaitContainer() {
}

void CConnectWaitContainer::ConstructL() {
}

void CConnectWaitContainer::DialogDismissedL(TInt aButtonId) {
}

TBool CConnectWaitContainer::IsProcessDone() const {
	if (iConn->iAborted) {
		return ETrue;
	}
	const char *err;
	int retval = iConn->ready(&err);
	if (retval == 0)
		return EFalse;
	if (retval < 0) {
		internalShowWarning(err);
	}
	return ETrue;
}

void CConnectWaitContainer::ProcessFinished() {
}

void CConnectWaitContainer::StepL() {
}



Connection *showDialogL(TBool aClient) {
	CBTConnection *conn = CBTConnection::NewLC();
	if (aClient)
		conn->ConnectL();
	else
		conn->AcceptL();

	CConnectWaitContainer *container = CConnectWaitContainer::NewL(conn);
	CleanupStack::PushL(container);
	CAknWaitNoteWrapper* wrapper = CAknWaitNoteWrapper::NewL();

	CleanupStack::PushL(reinterpret_cast<CBase*>(wrapper));
	TBool success = wrapper->ExecuteL((aClient ? R_BANTUMI_WAITING_CLIENT : R_BANTUMI_WAITING_SERVER), *container, ETrue);
	CleanupStack::PopAndDestroy(wrapper);
	CleanupStack::PopAndDestroy(container);

	restartTimer();

	if (!success) {
		CleanupStack::PopAndDestroy(conn);
		return NULL;
	}

	if (conn->iAborted) {
		CleanupStack::PopAndDestroy(conn);
		return NULL;
	}
		
	const char *err;
	if (conn->ready(&err) > 0) {
		CleanupStack::Pop(conn);
		return conn;
	}

	CleanupStack::PopAndDestroy(conn);
	return NULL;
}

Connection *showConnectDialog() {
	Connection *conn = NULL;
	TRAPD(err, conn = showDialogL(ETrue));
	if (err != KErrNone) {
		internalShowWarning(err);
		restartTimer();
		return NULL;
	}
	return conn;
}


Connection *showAcceptDialog() {
	Connection *conn = NULL;
	TRAPD(err, conn = showDialogL(EFalse));
	if (err != KErrNone) {
		internalShowWarning(err);
		restartTimer();
		return NULL;
	}
	return conn;
}



