/*
    Bantumi
    Copyright 2005, 2006 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsj
    martin@martin.st
*/

#include <stdio.h>
#include <stdlib.h>
#include "gui.h"
#include "tcpip.h"
#include <gtk/gtk.h>
#include <string.h>

void initGui() {
	gtk_init(NULL, NULL);
}

static void responseDialog(GtkDialog *dialog, gint arg1, gpointer data) {
	gtk_widget_destroy(GTK_WIDGET(dialog));
	gtk_main_quit();
}

void showWarning(const char *str) {
	GtkWidget *dialog = gtk_message_dialog_new(NULL, (GtkDialogFlags) 0, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, "%s", str);
	g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(responseDialog), NULL);
	gtk_widget_show_all(dialog);
	gtk_main();
}


static GtkWidget *connect;
static GtkEntry *hostEntry;
static GtkEntry *portEntry;
static TCPConnection *conn;
static GtkProgressBar *progress;
static char lastHost[200] = "";
static int lastPort = DEFAULT_TCP_PORT;
static bool keepConn;
static guint timer;

static void destroy(GtkWidget *widget, gpointer data) {
	if (!keepConn) {
		delete conn;
		conn = NULL;
	}
	if (timer > 0)
		g_source_remove(timer);
	gtk_main_quit();
}

static gboolean timeoutFunc(gpointer data) {
	if (conn) {
		const char *err;
		int retval;
		if (hostEntry)
			retval = conn->pollConnect(0, &err);
		else
			retval = conn->pollAccept(0, &err);
		if (retval > 0) {
			keepConn = true;
			timer = 0;
			gtk_widget_destroy(GTK_WIDGET(data));
			return FALSE;
		} else if (retval < 0) {
			showWarning(err);
			delete conn;
			conn = NULL;
			gtk_progress_bar_set_fraction(progress, 0);
			gtk_widget_set_sensitive(connect, TRUE);
			if (hostEntry)
				gtk_widget_set_sensitive(GTK_WIDGET(hostEntry), TRUE);
			gtk_widget_set_sensitive(GTK_WIDGET(portEntry), TRUE);
			timer = 0;
			return FALSE;
		} else {
			gtk_progress_bar_pulse(progress);
			return TRUE;
		}
	} else {
		timer = 0;
		return FALSE;
	}
}

static void click(GtkWidget *widget, gpointer data) {
	if (widget == connect) {
		const char *host = NULL;
		if (hostEntry) {
			host = gtk_entry_get_text(hostEntry);
			strncpy(lastHost, host, sizeof(lastHost)-1);
			lastHost[sizeof(lastHost)-1] = '\0';
		}
		int port = atoi(gtk_entry_get_text(portEntry));
		lastPort = port;
		const char *err;
		if (hostEntry)
			conn = TCPConnection::startConnect(host, port, &err);
		else
			conn = TCPConnection::startAccept(port, &err);
		if (conn) {
			timer = g_timeout_add(30, timeoutFunc, data);
			gtk_widget_set_sensitive(connect, FALSE);
			if (hostEntry)
				gtk_widget_set_sensitive(GTK_WIDGET(hostEntry), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(portEntry), FALSE);
			gtk_progress_bar_pulse(progress);
		} else {
			showWarning(err);
		}
	} else {
		if (conn) {
			delete conn;
			conn = NULL;
			gtk_progress_bar_set_fraction(progress, 0);
			gtk_widget_set_sensitive(connect, TRUE);
			if (hostEntry)
				gtk_widget_set_sensitive(GTK_WIDGET(hostEntry), TRUE);
			gtk_widget_set_sensitive(GTK_WIDGET(portEntry), TRUE);
			if (timer > 0)
				g_source_remove(timer);
			timer = 0;
		} else {
			gtk_widget_destroy(GTK_WIDGET(data));
		}
	}
}


Connection *showDialog(bool host) {
	GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	const char *title = "Bantumi GL - Connect";
	if (host)
		title = "Bantumi GL - Host";
	gtk_window_set_title(GTK_WINDOW(window), title);
	g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(destroy), NULL);

	gtk_container_set_border_width(GTK_CONTAINER(window), 10);

	GtkContainer *vbox = GTK_CONTAINER(gtk_vbox_new(FALSE, 10));
	gtk_container_add(GTK_CONTAINER(window), GTK_WIDGET(vbox));

	GtkWidget *label2 = gtk_label_new("Server port:");
	portEntry = GTK_ENTRY(gtk_entry_new());

	char buf[50];
	sprintf(buf, "%d", lastPort);
	gtk_entry_set_text(portEntry, buf);

	GtkTable *table = GTK_TABLE(gtk_table_new(2, 2, FALSE));
	gtk_table_set_row_spacings(table, 10);
	gtk_table_set_col_spacings(table, 10);
	if (!host) {
		GtkWidget *label1 = gtk_label_new("Server host:");
		hostEntry = GTK_ENTRY(gtk_entry_new());
		gtk_entry_set_text(hostEntry, lastHost);
		gtk_table_attach_defaults(table, label1, 0, 1, 0, 1);
		gtk_table_attach_defaults(table, GTK_WIDGET(hostEntry), 1, 2, 0, 1);
	} else {
		hostEntry = NULL;
	}
	gtk_table_attach_defaults(table, label2, 0, 1, 1, 2);
	gtk_table_attach_defaults(table, GTK_WIDGET(portEntry), 1, 2, 1, 2);
	gtk_container_add(vbox, GTK_WIDGET(table));

	gtk_container_add(vbox, gtk_hseparator_new());

	GtkContainer *hbox = GTK_CONTAINER(gtk_hbox_new(FALSE, 10));

	progress = GTK_PROGRESS_BAR(gtk_progress_bar_new());
	gtk_progress_bar_set_pulse_step(progress, 0.03);

	if (host) {
		connect = gtk_button_new_with_label("Host");
		gtk_button_set_image(GTK_BUTTON(connect), gtk_image_new_from_stock(GTK_STOCK_CONNECT, GTK_ICON_SIZE_BUTTON));
//		gtk_button_set_label(GTK_BUTTON(connect), "Host");
	} else {
		connect = gtk_button_new_from_stock(GTK_STOCK_CONNECT);
	}
	g_signal_connect(G_OBJECT(connect), "clicked", G_CALLBACK(click), window);
	GtkWidget *cancel = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
	g_signal_connect(G_OBJECT(cancel), "clicked", G_CALLBACK(click), window);
	gtk_container_add(GTK_CONTAINER(hbox), GTK_WIDGET(progress));
	gtk_container_add(GTK_CONTAINER(hbox), connect);
	gtk_container_add(GTK_CONTAINER(hbox), cancel);

	gtk_container_add(vbox, GTK_WIDGET(hbox));

	gtk_widget_show_all(window);

	conn = NULL;
	keepConn = false;
	timer = 0;
	gtk_main();

	return conn;
}

Connection *showConnectDialog() {
	return showDialog(false);
}

Connection *showAcceptDialog() {
	return showDialog(true);
}



